#
# preload.py: PreloadImageCreator class for creating preload image.
#
# Copyright 2019, NFS China, Inc.
#
# lina@cpu-os.ac.cn
#

import sys
import os
import os.path
#import lzo
import commands

from imgcreate.errors import *
from imgcreate.fs import *
from imgcreate.creator import *
from hashlib import md5

class PreloadImageCreator(LoopImageCreator):

    def __init__(self, ks, name, fslabel=None, releasever=None, tmpdir="/tmp",
                 title="Linux", product="Linux", useplugins=False, cacheonly=False,
                 docleanup=True, arch=None, runopts=None):

        LoopImageCreator.__init__(self, ks, name,
                                  fslabel=fslabel,
                                  releasever=releasever,
                                  tmpdir=tmpdir,
                                  useplugins=useplugins,
                                  cacheonly=cacheonly,
                                  docleanup=docleanup)

        """ The preload image compressed by lzo"""
        self.compress_type = "lzo"

        self.skip_compression = False

        self.skip_minimize = False

        self._timeout = kickstart.get_timeout(self.ks, 10)

        self.default_kernel = kickstart.get_default_kernel(self.ks, "kernel")

        self.arch = arch

        self.run_options = runopts

        self.isomode = 'oem'

    def _create_bootconfig(self):
        self._update_sys_files()

    def _update_sys_files(self):
        if self.isomode == "oem":
            cdspec = "/usr/cdspec-oem"
        else:
            cdspec = "/usr/cdspec-live"

        if os.path.exists("%s/s3g_driver_install.sh" % cdspec):
            logging.info("Copying %s/s3g_driver_install.sh to %s/usr/local/sbin/" % (cdspec, self._instroot))
            shutil.copy("%s/s3g_driver_install.sh" % cdspec, "%s/usr/local/sbin/s3g_driver_install.sh" % self._instroot)

        if os.path.exists("%s/trusted.gpg" % cdspec):
            logging.info("Copying %s/trusted.gpg to %s/etc/apt/" % (cdspec, self._instroot))
            shutil.copy("%s/trusted.gpg" % cdspec, "%s/etc/apt/trusted.gpg" % self._instroot)

    def _stage_final_image(self):
        # remove excluded_pkgs
        #excludepkgs = self._get_excluded_packages()
        #self._remove_exclude_pkgs(excludepkgs)

        # create filesystem.manifest
        cmd = "chroot %s dpkg-query -W --showformat='${Package} ${Version}\n'" % (self._instroot)
        pkgList = commands.getstatusoutput(cmd)
        pkgList = pkgList[1]
        manifestfile = self._outdir + '/filesystem.manifest.log'
        Fp = open(manifestfile, "w+")
        Fp.write(pkgList)
        Fp.close()
        shutil.move(manifestfile, self._outdir + "/../../")

        # exec oem ship
        self._exec_oem_ship()

        # create md5
        self._generate_md5_checkfile()

        # create osbase.lzo
        os.chdir(self._instroot)
        tarfile = self._outdir + "/osbase.tar"

        # collect tarfiles
        all_tar = []
        excludedirs = ['debootstrap', 'dev', 'proc', 'sys', 'run', 'mnt', 'vmlinuz.old', 'initrd.img.old']
        for dirs in os.listdir(self._instroot):
            if dirs not in excludedirs:
                all_tar.append(dirs)

        tarcmd = ['tar', '-cf', tarfile]
        tarcmd.extend(all_tar)
        try:
            p = subprocess.Popen(tarcmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            (output, error) = p.communicate()
        except OSError, error:
            raise CreatorError("Failed to create tar file. %s" % error)

        try:
            osbasefile = self._outdir + '/osbase.lzo'
            p = subprocess.Popen(['lzop', tarfile, '-o', osbasefile], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            (output, error) = p.communicate()
        except OSError, error:
            raise CreatorError("Failed to create lzo file: %s" % error)

        logging.info("Create LZO succeed.")
        shutil.move(osbasefile, self._outdir + "/../../")
        os.chdir(self._outdir + "/../../")
        workpre_dir = os.getcwd()
        
        if self.docleanup:
            shutil.rmtree(workpre_dir + "/work-livecd", ignore_errors=True)
        return

    def _mount_instroot(self):
        self.base_on = True
        LoopImageCreator._mount_instroot(self, base_on)

    def _unmount_instroot(self):
        LoopImageCreator._unmount_instroot(self)

    def _get_required_packages(self):
         return ['oem-config','oem-config-gtk'] + \
                 LoopImageCreator._get_required_packages(self)

    def _get_excluded_packages(self):
        excludepkgs = ['casper', 
                       'ubiquity', 
                       'ubiquity-casper', 
                       'ubiquity-frontend-debconf', 
                       'ubiquity-frontend-gtk',
                       'ubiquity-ubuntu-artwork',
                       'ubiquity-settings',
                       'ubiquity-touch-settings',
                       'libdebian-installer4',
                       'user-setup']
        return excludepkgs + LoopImageCreator._get_excluded_packages(self)

    def _remove_exclude_pkgs(self, excludepkgs, in_chroot=True):
        pkglist = ' '.join(excludepkgs)
        script = "dpkg -P %s" % pkglist

        inchrootfile = "/tmp/post-remove-pkg"

        scriptfile = self._instroot + inchrootfile

        fp = open(scriptfile, 'w')
        fp.write(script)
        fp.close()
        os.chmod(scriptfile, 0755)
        
        if in_chroot:
            try:
                ecode = subprocess.Popen(['/bin/bash', inchrootfile], 
                                         stdout=subprocess.PIPE,
                                         stderr=subprocess.PIPE,
                                         preexec_fn=self._chroot)
                output, error = ecode.communicate()
            except OSError, error:
                raise CreatorError("Failed to remove exludepkgs %s in env, %s",(pkglist, error))

        os.remove(scriptfile)

    def _exec_oem_ship(self):

        cmd = "chroot %s /bin/bash -c \"/usr/sbin/oem-config-prepare\"" % self._instroot
        #try:
        #    os.system(cmd)
        #except OSError:
        #    raise CreatorError("Failed to Execute oem-config-prepare for ship")
        oemshipfile = "/usr/sbin/oem-config-prepare"
        cmd = ['/bin/bash', oemshipfile]
        output = self.do_chroot(cmd)
        logging.info("The output of do_chroot %s: %s" % (cmd, output))

        #oemshipfile = "/usr/sbin/oem-config-prepare"
        #if os.path.exists(self._instroot + oemshipfile):
        #    try:
        #        p = subprocess.Popen(['/bin/bash', oemshipfile], preexec_fn=self._chroot(), stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        #        (output, error) = p.communicate()
        #    except OSError, error:
        #        raise CreatorError("Failed to Execute oem-config-prepare for ship. %s" % error)
        #else:
        #    logging.info("%s not exist! Skip." % oemshipfile)


    def _generate_md5_checkfile(self, recursive=False):
        """ Calculate the md5hash for files in /bin, /sbin, /usr/bin, /usr/sbin
        you can also add other dirs."""
        
        targetdirs = ['/bin', '/sbin', '/usr/bin', '/usr/sbin']
        filelist = []
        checkedstr = ""


        for td in targetdirs:
            for rootp, dirs, files in os.walk(td):
                for name in files:
                    filelist.append(os.path.join(rootp, name))

        # check md5
        for f in filelist:
            targetfile = self._instroot + f
            if os.path.exists(targetfile):
                md5value = md5(open(targetfile, 'rb').read()).hexdigest()
                checkedstr += "%s  %s\n" % (f, md5value)

        md5checkfile = self._outdir + "/md5sum.log"
        fp = open(md5checkfile, 'w+')
        fp.write(checkedstr)
        fp.close()

        # copy to /tmp
        shutil.move(md5checkfile, self._outdir + "/../../")
